@@ -1,30 +1,27 @@
-import functools
 import inspect
 import logging
+import sys
 import traceback
 import typing
-from copy import deepcopy
 
 import interactions
 from . import utils
 from .command import MolterCommand
-from .context import HybridContext
 from .context import MolterContext
 from .converters import MolterConverter
-from .hybrid import _molter_from_slash
 from interactions import ext
-from interactions.client.decor import command as slash_command
 
 __all__ = (
     "__version__",
     "base",
     "MolterInjectedClient",
+    "MolterExtensionMixin",
     "MolterExtension",
-    "Molter",
+    "MolterManager",
     "setup",
 )
 
-__version__ = "0.5.0"
+__version__ = "0.6.0"
 
 logger: logging.Logger = logging.getLogger("molter")
 
@@ -43,45 +40,66 @@ base = ext.Base(
         " demonstration of molter-core."
     ),
     packages=["interactions.ext.molter"],
-    requirements=["discord-py-interactions>=4.2.0"],
+    requirements=["discord-py-interactions>=4.3.0"],
 )
 
 
 class MolterInjectedClient(interactions.Client):
     """
-    A semi-stub for Clients injected with Molter.
+    A semi-stub for Clients injected with molter.
     This should only be used for typehinting.
     """
 
-    molter: "Molter"
+    molter: "MolterManager"
 
 
-class MolterExtension(interactions.Extension):
-    """An extension that allows you to use molter commands in them."""
+class MolterExtensionMixin:
+    """
+    A mixin that can be used to add molter functionality into any subclass of
+    interactions.py's extensions. Simply use it like so:
+
+    ```python
+    class MyExt(MolterExtensionMixin, ActualExtension):
+        ...
+    ```
+    """
 
     client: interactions.Client
+    _error_callback: typing.Optional[
+        typing.Callable[[MolterContext, Exception], typing.Coroutine]
+    ] = None
     _molter_prefixed_commands: typing.List[MolterCommand]
 
-    def __new__(
-        cls, client: interactions.Client, *args, **kwargs
-    ) -> "interactions.Extension":
-        self: "MolterExtension" = super().__new__(cls, client, *args, **kwargs)  # type: ignore
+    def __new__(cls, client: interactions.Client, *args, **kwargs):
+        self = super().__new__(cls, client, *args, **kwargs)  # type: ignore
         self._molter_prefixed_commands = []
 
         # typehinting funkyness for better typehints
         self.client = typing.cast(MolterInjectedClient, self.client)
 
-        for _, cmd in inspect.getmembers(
+        error_handler_count = 0
+
+        for _, func in inspect.getmembers(
             self,
             predicate=lambda x: isinstance(x, MolterCommand)
-            or hasattr(x, "__molter_command__"),
+            or hasattr(x, "__ext_molter_error__"),
         ):
-            cmd: MolterCommand = getattr(cmd, "__molter_command__", None) or cmd
+            if isinstance(func, MolterCommand):
+                cmd: MolterCommand = func
 
-            if not cmd.is_subcommand():  # we don't want to add subcommands
-                cmd = utils._wrap_recursive(cmd, self)
-                self._molter_prefixed_commands.append(cmd)
-                self.client.molter.add_prefixed_command(cmd)
+                if not cmd.is_subcommand():  # we don't want to add subcommands
+                    cmd = utils._wrap_recursive(cmd, self)
+                    self._molter_prefixed_commands.append(cmd)
+                    self.client.molter.add_prefixed_command(cmd)
+            elif hasattr(func, "__ext_molter_error__"):
+                if error_handler_count >= 1:
+                    raise ValueError(
+                        "A molter extension cannot have more than one molter command"
+                        " error handler."
+                    )
+
+                self._error_callback = func
+                error_handler_count += 1
 
         return self
 
@@ -96,10 +114,16 @@ class MolterExtension(interactions.Extension):
             for name in names_to_remove:
                 self.client.molter.prefixed_commands.pop(name, None)
 
-        return await super().teardown(*args, **kwargs)
+        return await super().teardown(*args, **kwargs)  # type: ignore
 
 
-class Molter:
+class MolterExtension(MolterExtensionMixin, interactions.Extension):
+    """An extension that allows you to use molter commands in them."""
+
+    pass
+
+
+class MolterManager:
     """
     The main part of the extension. Deals with injecting itself in the first place.
 
@@ -110,11 +134,6 @@ class Molter:
         generate_prefixes (`typing.Callable`, optional): An asynchronous function \
             that takes in a `Client` and `Message` object and returns either a \
             string or an iterable of strings. Defaults to None.
-        fetch_data_for_context (`bool`): If molter should attempt to fetch extra \
-            data, like the `Guild` and `Channel` where the message was sent. \
-            Turning this on may make the bot respond slower or faster depending on \
-            the converters used in the command, but usually is slower. \
-            Defaults to False.
         on_molter_command_error (`typing.Callable`, optional): An asynchronous function \
             that takes in a `MolterContext` and `Exception` to handle errors that occur \
             when running molter commands. By default, molter will output the error to \
@@ -128,6 +147,7 @@ class Molter:
     def __init__(
         self,
         client: interactions.Client,
+        *,
         default_prefix: typing.Optional[typing.Union[str, typing.Iterable[str]]] = None,
         generate_prefixes: typing.Optional[
             typing.Callable[
@@ -137,18 +157,15 @@ class Molter:
                 ],
             ]
         ] = None,
-        fetch_data_for_context: bool = False,
         on_molter_command_error: typing.Optional[
             typing.Callable[[MolterContext, Exception], typing.Coroutine]
         ] = None,
     ) -> None:
-
         # typehinting funkyness for better typehints
         client = typing.cast(MolterInjectedClient, client)
 
         self.client = client
         self.default_prefix = default_prefix
-        self.fetch_data_for_context = fetch_data_for_context
         self.prefixed_commands: typing.Dict[str, MolterCommand] = {}
 
         if default_prefix is None and generate_prefixes is None:
@@ -169,32 +186,82 @@ class Molter:
         # this allows us to use a (hopefully) non-conflicting namespace
         self.client.molter = self
 
+        # i hope someone dies internally when looking at this /lhj
+        # but the general idea is that we want to process commands
+        # from the main file right before starting up, so that we have a full
+        # list of commands (otherwise, we may only get a partial list)
+
+        # since we can pretty much guarantee that the ready function will only be
+        # ran at the end for a variety of reasons, we can hook onto this
+        # rather easily without many problems... unless someone's adding commands
+        # after the bot has started, to which they should be using add_prefixed_command
+        # anyways.
+        def cursed_override(old_ready):
+            async def new_ready():
+                for _, func in inspect.getmembers(
+                    sys.modules["__main__"],
+                    predicate=lambda x: isinstance(x, MolterCommand)
+                    and not x.is_subcommand(),
+                ):
+                    self.add_prefixed_command(func)
+                await old_ready()
+
+            return new_ready
+
+        self.client._ready = cursed_override(self.client._ready)
+
         self.client.event(self._handle_prefixed_commands, name="on_message_create")  # type: ignore
         self.client.event(self.on_molter_command_error, name="on_molter_command_error")  # type: ignore
 
     def add_prefixed_command(self, command: MolterCommand) -> None:
-        """Add a prefixed command to the client.
+        """
+        Add a prefixed command to the client.
 
         Args:
             command (`MolterCommand`): The command to add.
         """
-        if command.parent:
-            return  # silent return to ignore subcommands - hacky, ik
-
-        if command.name not in self.prefixed_commands:
-            self.prefixed_commands[command.name] = command
-        else:
+        if command.is_subcommand():
             raise ValueError(
-                f"Duplicate Command! Multiple commands share the name {command.name}"
+                "You cannot add subcommands to the client - add the base command"
+                " instead."
             )
 
+        command._parse_parameters()
+
+        if self.prefixed_commands.get(command.name):
+            raise ValueError(
+                "Duplicate command! Multiple commands share the name/alias:"
+                f" {command.name}."
+            )
+        self.prefixed_commands[command.name] = command
+
         for alias in command.aliases:
-            if alias not in self.prefixed_commands:
-                self.prefixed_commands[alias] = command
-                continue
-            raise ValueError(
-                f"Duplicate Command! Multiple commands share the name/alias {alias}"
-            )
+            if self.prefixed_commands.get(alias):
+                raise ValueError(
+                    "Duplicate command! Multiple commands share the name/alias:"
+                    f" {alias}."
+                )
+            self.prefixed_commands[alias] = command
+
+    def remove_prefixed_command(self, name: str):
+        """
+        Removes a command if it exists.
+        If an alias is specified, only the alias will be removed.
+
+        Args:
+            name (`str`): The command to remove.
+        """
+        command = self.prefixed_commands.pop(name, None)
+
+        if command is None:
+            return
+
+        if name in command.aliases:
+            command.aliases.remove(name)
+            return
+
+        for alias in command.aliases:
+            self.prefixed_commands.pop(alias, None)
 
     def prefixed_command(
         self,
@@ -212,6 +279,10 @@ class Molter:
         ] = None,
     ) -> typing.Callable[..., MolterCommand]:
         """
+        WARNING:
+            This method is deprecated and will be removed in a future release.
+            Please use `molter.prefixed_command` instead.
+
         A decorator to declare a coroutine as a Molter prefixed command.
 
         Parameters:
@@ -254,7 +325,13 @@ class Molter:
         """
 
         def wrapper(func):
-            cmd = MolterCommand(  # type: ignore
+            logger.warning(
+                "`MolterManager.prefixed_command` (commonly seen as"
+                " `@molt.prefixed_command`) and its aliases has been deprecated. Use"
+                " `molter.prefixed_command` or its aliases instead."
+            )
+
+            return MolterCommand(
                 callback=func,
                 name=name or func.__name__,
                 aliases=aliases or [],
@@ -267,83 +344,12 @@ class Molter:
                 type_to_converter=type_to_converter  # type: ignore
                 or getattr(func, "_type_to_converter", {}),
             )
-            self.add_prefixed_command(cmd)
-            return cmd
 
         return wrapper
 
     prefix_command = prefixed_command
     text_based_command = prefixed_command
 
-    @functools.wraps(slash_command)
-    def hybrid_slash(
-        self,
-        *,
-        name: typing.Optional[str] = interactions.MISSING,  # type: ignore
-        description: typing.Optional[str] = interactions.MISSING,  # type: ignore
-        scope: typing.Optional[
-            typing.Union[
-                int,
-                interactions.Guild,
-                typing.List[int],
-                typing.List[interactions.Guild],
-            ]
-        ] = interactions.MISSING,  # type: ignore
-        options: typing.Optional[
-            typing.Union[
-                typing.Dict[str, typing.Any],
-                typing.List[typing.Dict[str, typing.Any]],
-                interactions.Option,
-                typing.List[interactions.Option],
-            ]
-        ] = interactions.MISSING,  # type: ignore
-        name_localizations: typing.Optional[
-            typing.Dict[typing.Union[str, interactions.Locale], str]
-        ] = interactions.MISSING,  # type: ignore
-        description_localizations: typing.Optional[
-            typing.Dict[typing.Union[str, interactions.Locale], str]
-        ] = interactions.MISSING,  # type: ignore
-        default_member_permissions: typing.Optional[
-            typing.Union[int, interactions.Permissions]
-        ] = interactions.MISSING,  # type: ignore
-        dm_permission: typing.Optional[bool] = interactions.MISSING,  # type: ignore
-    ):
-        """
-        A decorator for creating hybrid commands based off a normal slash command.
-        Uses all normal slash command arguments (besides for type), but also makes
-        a prefixed command when used in conjunction with `MolterExtension`.
-
-        Remember to use `HybridContext` as the context for proper type hinting.
-        Subcommand options do not work with this decorator right now.
-        """
-        kwargs = locals()
-        del kwargs["self"]
-        kwargs["type"] = interactions.ApplicationCommandType.CHAT_INPUT
-
-        def decorator(coro):
-            coro_copy = deepcopy(coro)
-            molt_cmd = _molter_from_slash(coro_copy, **kwargs)
-            self.add_prefixed_command(molt_cmd)
-
-            async def wrapped_command(
-                ctx: interactions.CommandContext, *args, **kwargs
-            ):
-                new_ctx = HybridContext(
-                    message=ctx.message,
-                    user=ctx.user,
-                    member=ctx.member,
-                    channel=ctx.channel,
-                    guild=ctx.guild,
-                    prefix="/",
-                    command_context=ctx,
-                )
-                new_ctx.args = list(args) + list(kwargs.values())
-                await coro(new_ctx, *args, **kwargs)
-
-            return self.client.command(**kwargs)(wrapped_command)
-
-        return decorator
-
     async def generate_prefixes(
         self, client: interactions.Client, msg: interactions.Message
     ) -> typing.Union[str, typing.Iterable[str]]:
@@ -394,14 +400,18 @@ class Molter:
         # weirdly enough, sometimes this isn't set right
         msg._client = self.client._http
 
-        channel = None
-        guild = None
+        channel = await interactions.get(
+            self.client, interactions.Channel, object_id=int(msg.channel_id)
+        )
 
-        if self.fetch_data_for_context:
-            # get from cache if possible
-            channel = await msg.get_channel()
-            if msg.guild_id:
-                guild = await msg.get_guild()
+        if (guild_id := msg.guild_id) or (guild_id := channel.guild_id):
+            guild = await utils._wrap_lib_exception(
+                interactions.get(
+                    self.client, interactions.Guild, object_id=int(guild_id)
+                )
+            )
+        else:
+            guild = None
 
         return MolterContext(  # type: ignore
             client=self.client,
@@ -412,28 +422,6 @@ class Molter:
             guild=guild,
         )
 
-    def _standard_to_hybrid(self, ctx: MolterContext) -> HybridContext:
-        """
-        Creates a `HybridContext` object from `MolterContext`.
-
-        Args:
-            ctx (`MolterContext`): The context to create a hybrid context from.
-
-        Returns:
-            `HybridContext`: The context generated.
-        """
-        new_ctx = HybridContext(  # type: ignore
-            client=ctx.client,
-            message=ctx.message,
-            user=ctx.author,  # type: ignore
-            member=ctx.member,
-            channel=ctx.channel,
-            guild=ctx.guild,
-        )
-        new_ctx.prefix = ctx.prefix
-        new_ctx.content_parameters = ctx.content_parameters
-        return new_ctx
-
     async def _handle_prefixed_commands(self, msg: interactions.Message):
         """
         Determines if a command is being triggered and dispatch it.
@@ -458,9 +446,7 @@ class Molter:
             context = await self._create_context(msg)
             context.prefix = prefix_used
             context.content_parameters = utils.remove_prefix(msg.content, prefix_used)
-            command: typing.Optional[
-                typing.Union[Molter, MolterCommand]
-            ] = self.client.molter
+            command = self.client.molter
 
             while True:
                 first_word: str = utils.get_first_word(context.content_parameters)  # type: ignore
@@ -480,18 +466,20 @@ class Molter:
                     try:
                         await command._run_checks(context)
                     except Exception as e:
-                        self.client._websocket._dispatch.dispatch(
-                            "on_molter_command_error", context, e
-                        )
+                        if command.error_callback:
+                            await command.error_callback(context, e)  # type: ignore
+                        elif command.extension and command.extension._error_callback:
+                            await command.extension._error_callback(context, e)
+                        else:
+                            self.client._websocket._dispatch.dispatch(
+                                "on_molter_command_error", context, e
+                            )
                         return
 
-            if isinstance(command, Molter):
+            if isinstance(command, MolterManager):
                 command = None
 
             if command and command.enabled:
-                if command.hybrid:
-                    context = self._standard_to_hybrid(context)
-
                 # this looks ugly, ik
                 context.invoked_name = utils.remove_suffix(
                     utils.remove_prefix(msg.content, prefix_used),
@@ -501,19 +489,28 @@ class Molter:
                 context.command = command
 
                 try:
-                    await command(context)
-                except Exception as e:
-                    self.client._websocket._dispatch.dispatch(
-                        "on_molter_command_error", context, e
-                    )
-                finally:
                     self.client._websocket._dispatch.dispatch(
                         "on_molter_command", context
                     )
+                    await command(context)
+                except Exception as e:
+                    if command.error_callback:
+                        await command.error_callback(context, e)  # type: ignore
+                    elif command.extension and command.extension._error_callback:
+                        await command.extension._error_callback(context, e)
+                    else:
+                        self.client._websocket._dispatch.dispatch(
+                            "on_molter_command_error", context, e
+                        )
+                finally:
+                    self.client._websocket._dispatch.dispatch(
+                        "on_molter_command_complete", context
+                    )
 
 
 def setup(
     client: interactions.Client,
+    *,
     default_prefix: typing.Optional[typing.Union[str, typing.Iterable[str]]] = None,
     generate_prefixes: typing.Optional[
         typing.Callable[
@@ -523,15 +520,14 @@ def setup(
             ],
         ]
     ] = None,
-    fetch_data_for_context: bool = False,
     on_molter_command_error: typing.Optional[
         typing.Callable[[MolterContext, Exception], typing.Coroutine]
     ] = None,
-    *args,
     **kwargs,
-) -> Molter:
+) -> MolterManager:
     """
-    Allows setup of Molter through normal extension loaded.
+    Allows setup of Molter through normal extension loading.
+    It is recommended to use this function directly, though.
 
     Parameters:
         client (`interactions.Client`): The client instance.
@@ -540,11 +536,6 @@ def setup(
         generate_prefixes (`typing.Callable`, optional): An asynchronous function \
             that takes in a `Client` and `Message` object and returns either a \
             string or an iterable of strings. Defaults to None.
-        fetch_data_for_context (`bool`): If molter should attempt to fetch extra \
-            data, like the `Guild` and `Channel` where the message was sent. \
-            Turning this on may make the bot respond slower or faster depending on \
-            the converters used in the command, but usually is slower. \
-            Defaults to False.
         on_molter_command_error (`typing.Callable`, optional): An asynchronous function \
             that takes in a `MolterContext` and `Exception` to handle errors that occur \
             when running molter commands. By default, molter will output the error to \
@@ -555,12 +546,12 @@ def setup(
         defaults to using it being mentioned as its prefix.
 
     Returns:
-        `Molter`: The class that deals with all things Molter.
+        `MolterManager`: The class that deals with all things Molter.
     """
-    return Molter(
-        client,
-        default_prefix,
-        generate_prefixes,
-        fetch_data_for_context,
-        on_molter_command_error,
+    return MolterManager(
+        client=client,
+        default_prefix=default_prefix,
+        generate_prefixes=generate_prefixes,
+        on_molter_command_error=on_molter_command_error,
+        **kwargs,
     )
