@@ -11,7 +11,6 @@ import interactions
 from . import context
 from . import converters
 from . import errors
-from .utils import _qualname_wrap
 from .utils import _start_quotes
 from .utils import maybe_coroutine
 
@@ -19,11 +18,18 @@ __all__ = (
     "MolterCommand",
     "prefixed_command",
     "prefix_command",
+    "text_command",
     "text_based_command",
     "register_converter",
     "globally_register_converter",
+    "prefixed_parameter",
+    "ext_error_handler",
 )
 
+
+if typing.TYPE_CHECKING:
+    from .base import MolterExtension
+
 # 3.8+ compatibility
 NoneType = type(None)
 
@@ -38,6 +44,24 @@ except ImportError:  # 3.8-3.9
 # thankfully, modules are singletons
 _global_type_to_converter = dict(converters.INTER_OBJECT_TO_CONVERTER)
 
+ERROR_HANDLER = typing.Callable[
+    [context.MolterContext, Exception], typing.Awaitable[typing.Any]
+]
+EXT_ERROR_HANDLER = typing.Callable[
+    [typing.Any, context.MolterContext, Exception], typing.Awaitable[typing.Any]
+]
+# for some reason, typing an ext here doesn't work, so oh well
+ERROR_HANDLER_FUNCTION = typing.Union[ERROR_HANDLER, EXT_ERROR_HANDLER]
+
+EEH = typing.TypeVar("EEH", bound=EXT_ERROR_HANDLER)
+EHF = typing.TypeVar("EHF", bound=ERROR_HANDLER_FUNCTION)
+
+
+class ParameterInfo(typing.TypedDict):
+    default: typing.Union[
+        typing.Callable[[context.MolterContext, str], typing.Any], typing.Any
+    ]
+
 
 @attrs.define(slots=True)
 class PrefixedCommandParameter:
@@ -48,7 +72,7 @@ class PrefixedCommandParameter:
 
     name: str = attrs.field(default=None)
     "The name of the parameter."
-    default: typing.Optional[typing.Any] = attrs.field(default=None)
+    default: typing.Any = attrs.field(default=interactions.MISSING)
     "The default value of the parameter."
     type: typing.Type = attrs.field(default=None)
     "The type of the parameter."
@@ -88,7 +112,7 @@ class _PrefixedArgsIterator:
     index: int = attrs.field(init=False, default=0)
     length: int = attrs.field(init=False, default=0)
 
-    def __iter__(self) -> "_PrefixedArgsIterator":
+    def __iter__(self) -> typing_extensions.Self:
         self.length = len(self.args)
         return self
 
@@ -266,6 +290,7 @@ def _greedy_parse(greedy: converters.Greedy, param: inspect.Parameter):
 def _get_params(
     signature: inspect.Signature,
     type_to_converter: typing.Dict[type, typing.Type[converters.MolterConverter]],
+    parameter_info: typing.Dict[str, ParameterInfo],
 ):
     cmd_params: list[PrefixedCommandParameter] = []
 
@@ -280,13 +305,18 @@ def _get_params(
 
         cmd_param = PrefixedCommandParameter()
         cmd_param.name = name
-        cmd_param.default = (
-            param.default if param.default is not param.empty else interactions.MISSING
-        )
         cmd_param.kind = param.kind
-
         cmd_param.type = anno = param.annotation
 
+        if parameter_dict := parameter_info.pop(name, None):
+            cmd_param.default = parameter_dict["default"]
+        else:
+            cmd_param.default = (
+                param.default
+                if param.default is not param.empty
+                else interactions.MISSING
+            )
+
         if typing_extensions.get_origin(anno) == converters.Greedy:
             anno, default = _greedy_parse(anno, param)
 
@@ -335,6 +365,11 @@ def _get_params(
 
         cmd_params.append(cmd_param)
 
+    if parameter_info:
+        for name in parameter_info.keys():
+            # this should only raise once due to how errors work
+            raise ValueError(f'"{name}" is not a parameter in the command provided.')
+
     return cmd_params
 
 
@@ -355,7 +390,11 @@ async def _convert(
     used_default = False
     if converted == interactions.MISSING:
         if param.optional:
-            converted = param.default
+            converted = (
+                await maybe_coroutine(param.default, ctx)
+                if inspect.isfunction(param.default)
+                else param.default
+            )
             used_default = True
         else:
             union_types = typing_extensions.get_args(param.type)
@@ -394,8 +433,12 @@ async def _greedy_convert(
             break
 
     if not greedy_args:
-        if param.default:
-            greedy_args = param.default  # im sorry, typehinters
+        if param.optional:
+            greedy_args = (
+                await maybe_coroutine(param.default, ctx)
+                if inspect.isfunction(param.default)
+                else param.default
+            )
         else:
             raise errors.BadArgument(
                 f"Failed to find any arguments for {repr(param.type)}."
@@ -410,7 +453,7 @@ async def _greedy_convert(
     hash=False,
 )
 class MolterCommand:
-    extension: typing.Any = attrs.field(default=None)
+    extension: typing.Optional["MolterExtension"] = attrs.field(default=None)
     "The extension this command belongs to."
     enabled: bool = attrs.field(default=True)
     "Whether this can be run at all."
@@ -441,18 +484,16 @@ class MolterCommand:
     """If `True` and if the base of a subcommand, every subcommand underneath
     it will run this command's checks and cooldowns before its own. Otherwise,
     only the subcommand's checks are checked."""
-    hybrid: bool = attrs.field(default=False)
-    """Is this command a hybrid command or not?"""
 
     help: typing.Optional[str] = attrs.field(default=None)
     """The long help text for the command."""
     brief: typing.Optional[str] = attrs.field(default=None)
     "The short help text for the command."
-    parent: typing.Optional["MolterCommand"] = attrs.field(
+    parent: typing.Optional[typing_extensions.Self] = attrs.field(
         default=None,
     )
     "The parent command, if applicable."
-    subcommands: typing.Dict[str, "MolterCommand"] = attrs.field(
+    subcommands: typing.Dict[str, typing_extensions.Self] = attrs.field(
         factory=dict,
     )
     "A dict of all subcommands for the command."
@@ -463,23 +504,24 @@ class MolterCommand:
         ]
     ] = attrs.field(factory=list)
     """A list of checks for this command."""
+    error_callback: typing.Optional[ERROR_HANDLER_FUNCTION] = attrs.field(default=None)
+    """The callback to use if an error occurs."""
 
     _usage: typing.Optional[str] = attrs.field(default=None, repr=False)
     _type_to_converter: typing.Dict[
         type, typing.Type[converters.MolterConverter]
     ] = attrs.field(factory=dict, converter=_merge_converters, repr=False)
+    _parameter_info: typing.Dict[str, ParameterInfo] = attrs.field(
+        factory=dict, repr=False
+    )
     _signature: typing.Optional[inspect.Signature] = attrs.field(
         default=None, repr=False
     )
 
     def __attrs_post_init__(self) -> None:
-        if not self._signature:
-            callback = _qualname_wrap(self.callback)
-            self._signature = inspect.signature(callback)
-
-        # doing this here just so we don't run into any issues here with a value
-        # not being there yet or something if we used defaults, idk
-        self.parameters = _get_params(self._signature, self._type_to_converter)
+        # for some reason, i can't seem to pick this up anywhere else
+        if hasattr(self.callback, "_parameter_info"):
+            self._parameter_info = self.callback._parameter_info
 
         # we have to do this afterwards as these rely on the callback
         # and its own value, which is impossible to get with attrs
@@ -528,7 +570,7 @@ class MolterCommand:
         return " ".join(name_deq)
 
     @property
-    def all_commands(self) -> typing.FrozenSet["MolterCommand"]:
+    def all_commands(self) -> typing.FrozenSet[typing_extensions.Self]:
         """Returns all unique subcommands underneath this command."""
         return frozenset(self.subcommands.values())
 
@@ -591,11 +633,38 @@ class MolterCommand:
 
         return " ".join(results)
 
+    def _parse_parameters(self) -> None:
+        """
+        Parses the parameters that this command has into a form molter can use.
+
+        This is purposely seperated like this to allow "lazy parsing" - parsing
+        as the command is added to a client rather than being parsed immediately.
+        This allows variables like "self" to be filtered out, and allows
+        for decorators that modify parameters to be parsed later, all at once.
+
+        This is called automatically if you add the command via
+        `MolterManager.add_prefixed_command`, but it is up to the user to remember
+        to put the extension in, if needed.
+        """
+
+        if not self._signature:
+            # we want to avoid ctx - self should have been handled earlier
+            callback = functools.partial(self.callback, None)
+            self._signature = inspect.signature(callback)
+
+        # doing this here just so we don't run into any issues here with a value
+        # not being there yet or something if we used defaults, idk
+        self.parameters = _get_params(
+            self._signature,
+            self._type_to_converter,
+            self._parameter_info,
+        )
+
     def is_subcommand(self) -> bool:
         """Returns if this command is a subcommand or not."""
         return bool(self.parent)
 
-    def add_command(self, cmd: "MolterCommand") -> None:
+    def add_command(self, cmd: typing_extensions.Self) -> None:
         """
         Adds a command as a subcommand to this command.
 
@@ -604,25 +673,24 @@ class MolterCommand:
         """
         cmd.parent = self  # just so we know this is a subcommand
 
-        cmd_names = frozenset(self.subcommands)
-        if cmd.name in cmd_names:
+        if self.subcommands.get(cmd.name):
             raise ValueError(
-                "Duplicate Command! Multiple commands share the name/alias"
-                f" `{self.qualified_name} {cmd.name}`"
+                "Duplicate command! Multiple commands share the name/alias:"
+                f" {self.qualified_name} {cmd.name}."
             )
         self.subcommands[cmd.name] = cmd
 
         for alias in cmd.aliases:
-            if alias in cmd_names:
+            if self.subcommands.get(alias):
                 raise ValueError(
-                    "Duplicate Command! Multiple commands share the name/alias"
-                    f" `{self.qualified_name} {cmd.name}`"
+                    "Duplicate command! Multiple commands share the name/alias:"
+                    f" {self.qualified_name} {cmd.name}."
                 )
             self.subcommands[alias] = cmd
 
     def remove_command(self, name: str) -> None:
         """
-        Removes a command as a subcommand from this command.
+        Removes a command as a subcommand from this command if it exists.
         If an alias is specified, only the alias will be removed.
 
         Args:
@@ -630,13 +698,17 @@ class MolterCommand:
         """
         command = self.subcommands.pop(name, None)
 
-        if command is None or name in command.aliases:
+        if command is None:
+            return
+
+        if name in command.aliases:
+            command.aliases.remove(name)
             return
 
         for alias in command.aliases:
             self.subcommands.pop(alias, None)
 
-    def get_command(self, name: str) -> typing.Optional["MolterCommand"]:
+    def get_command(self, name: str) -> typing.Optional[typing_extensions.Self]:
         """
         Gets a subcommand from this command. Can get subcommands of subcommands if needed.
         Args:
@@ -663,6 +735,37 @@ class MolterCommand:
 
         return cmd
 
+    @typing.overload
+    def error(self) -> typing.Callable[[EHF], EHF]:
+        ...
+
+    @typing.overload
+    def error(self, error_callback: EHF) -> EHF:
+        ...
+
+    def error(
+        self,
+        error_callback: typing.Optional[EHF] = None,
+    ) -> typing.Union[typing.Callable[[EHF], EHF], EHF]:
+        """
+        A decorator to register an error callback for a command.
+
+        Args:
+            error_callback (`Callable`):
+            The error callback to register. It must be an asynchronous function that
+            takes in MolterContext and an exception.
+        """
+
+        def decorator(error_callback: EHF):
+            self.error_callback = error_callback
+            return error_callback
+
+        if error_callback is not None:
+            self.error_callback = error_callback
+            return error_callback  # type: ignore
+
+        return decorator
+
     def subcommand(
         self,
         name: typing.Optional[str] = None,
@@ -678,7 +781,7 @@ class MolterCommand:
         type_to_converter: typing.Optional[
             typing.Dict[type, typing.Type[converters.MolterConverter]]
         ] = None,
-    ) -> typing.Callable[..., "MolterCommand"]:
+    ) -> typing.Callable[..., typing_extensions.Self]:
         """
         A decorator to declare a subcommand for a molter prefixed command.
 
@@ -845,13 +948,19 @@ class MolterCommand:
                             f"{param.name} is a required argument that is missing."
                         )
                     else:
+                        default = (
+                            await maybe_coroutine(param.default, ctx)
+                            if inspect.isfunction(param.default)
+                            else param.default
+                        )
+
                         if param.kind in {
                             inspect.Parameter.POSITIONAL_ONLY,
                             inspect.Parameter.VAR_POSITIONAL,
                         }:
-                            new_args.append(param.default)
+                            new_args.append(default)
                         else:
-                            kwargs[param.name] = param.default
+                            kwargs[param.name] = default
                             break
             elif not self.ignore_extra and not args.finished:
                 raise errors.BadArgument(f"Too many arguments passed to {self.name}.")
@@ -941,6 +1050,7 @@ def prefixed_command(
 
 
 prefix_command = prefixed_command
+text_command = prefixed_command
 text_based_command = prefixed_command
 
 
@@ -971,26 +1081,6 @@ def register_converter(
         else:
             command._type_to_converter = {type_: converter}
 
-        if isinstance(command, MolterCommand):
-            # we want to update any instance where the anno_type was used
-            # to use the provided converter without re-analyzing every param
-            for param in command.parameters:
-                param_type = param.type
-                if type_ == param_type:
-                    param.converters = [converter]
-                else:
-                    if (
-                        typing_extensions.get_origin(param_type)
-                        == typing_extensions.Annotated
-                    ):
-                        param_type = _get_from_anno_type(param_type)
-
-                    with contextlib.suppress(ValueError):
-                        # if you have multiple of the same anno/type here, i don't know
-                        # what to tell you other than why
-                        index = typing_extensions.get_args(param.type).index(type_)
-                        param.converters[index] = converter
-
         return command
 
     return wrapper
@@ -1013,3 +1103,83 @@ def globally_register_converter(
     # hate me, but i think it makes sense here
     global _global_type_to_converter
     _global_type_to_converter.update({anno_type: converter})
+
+
+def prefixed_parameter(
+    parameter_name: str,
+    *,
+    default: typing.Any = interactions.MISSING,
+    factory: typing.Optional[
+        typing.Callable[[context.MolterContext], typing.Any]
+    ] = None,
+) -> typing.Callable[..., MCT]:
+    """
+    A decorator that allows you to adjust a parameter in a prefixed comamnd.
+
+    Args:
+        parameter_name (`str`): The parameter in the command to edit.
+        default (`Any`, optional): The default value the parameter should have.
+        factory (`Callable[MolterContext]`, optional): A factory function to use
+        if no argument is provided for this parameter. The function can be asynchronous
+        or synchronous, and should take in solely a `MolterContext`.
+
+    Returns:
+        `Callable | MolterCommand`: Either the callback or the command.
+        If this is used after using the `molter.prefixed_command` decorator, it will be a command.
+        Otherwise, it will be a callback.
+    """
+
+    def wrapper(command: MCT) -> MCT:
+        parameter_info: ParameterInfo = {}  # type: ignore
+
+        if factory is not None and default is not interactions.MISSING:
+            raise ValueError("You cannot provide both a default and a factory.")
+        elif factory is None and default is interactions.MISSING:
+            raise ValueError("You must provide either a default or a factory.")
+
+        if factory is not None:
+            parameter_info["default"] = factory
+        elif default is not interactions.MISSING:
+            parameter_info["default"] = default
+
+        if hasattr(command, "_parameters_info"):
+            command._parameter_info[parameter_name] = parameter_info  # type: ignore
+        else:
+            command._parameter_info = {parameter_name: parameter_info}  # type: ignore
+
+        return command
+
+    return wrapper
+
+
+@typing.overload
+def ext_error_handler() -> typing.Callable[[EEH], EEH]:
+    ...
+
+
+@typing.overload
+def ext_error_handler(error_callback: EEH) -> EEH:
+    ...
+
+
+def ext_error_handler(
+    error_callback: typing.Optional[EEH] = None,
+) -> typing.Union[typing.Callable[[EEH], EEH], EEH]:
+    """
+    A decorator to register an error callback for a MolterExtension.
+
+    Args:
+        error_callback (`Callable`):
+        The error callback to register. It must be an asynchronous function that
+        takes in `self`, MolterContext and an exception.
+    """
+
+    def decorator(error_callback: EEH):
+        error_callback.__ext_molter_error__ = True
+        return error_callback
+
+    if error_callback is not None:
+        error_callback.__ext_molter_error__ = True
+        return error_callback
+
+    return decorator
